%
%  IMAGE CODING EXPERIMENTS
%
%  Capabilities of SPCA for nonlinear ICA or optimal quantization using different 
%  metrics can be assessed in coding image texture.
%  To illustrate this point we consider 2*10^5 image patches of size 2x2 
%  (a 4d problem) from the McGill calibrated natural image database
%  [Olmos04], and we make two experiments:
%
%  (1) STATISTICAL INDEPENDENCE  
%
%      We train SPCA using gamma = 1 and 1/3 on this database, and we
%      transform 3000 samples of such database with these criteria.
%      We also transform 2500 samples from the Barbara image (not included
%      in the training set).
%      
%      We evaluate the statistical relations in the transformed data in two
%      ways: (1) by measuring mutual information between transformed
%      coefficients (the standard way to assess independence), and (2) by 
%      inspecting the conditional probabilities on transformed coefficients
%      (a qualitative way to assess dependence proposed in [Simoncelli97] and
%      widely used thereafter).
%
%  (2) RECONSTRUCTION ERROR AFTER TRANSFORM QUANTIZATION
%
%      We quantize the transform of blocks of the Barbara image and invert 
%      the result back to the original spatial domain to assess the quality
%      of the quantized images.
%      We do that for the different considered domains (PCA, SPCA gamma=1, and
%      SPCA gamma=1/3) and for different quantization resolutions. 
%
%  CPU TIME ISSUES: given the fact that in this 4-d problem direct transform 
%  takes about 800 secs/sample and the inverse takes about 200 secs/sample
%  (in a Dell PowerEdge2900 processor), recomputing all the experiments in
%  the paper takes a lot of time. 
%  In these experiments we used 48 processors during 5 days!.
%
%  According to this, in this folder we provide the data and software to perform 
%  such experiments in a reduced-size scenario (18x18 patch of Barbara, i.e. 
%  81, 4-d samples). The interested reader may easily extend the experiment 
%  for the whole Barbara image and the whole database.
%  For the reader convenience, we also provide the computed results in our
%  experiments.
%
%  Software is organized in this way:
%
%     1- Initialization:  initialization_4d_texture_data.m
%
%     2- Transform of training data: 
%                  transform_training_data_criterion_2.m   XXX Usan folder!
%                  transform_training_data_criterion_3.m   XXX
%
%     3- Transform of test data: 
%                  transform_image_data_criterion_2.m
%                  transform_image_data_criterion_3.m
%
%     4- Quantization and inverse:
%                  quantize_and_invert_image_transf_crit_2.m
%                  quantize_and_invert_image_transf_crit_3.m
%
%     5- Analysis of transfrom coding results I 
%        (you can use this after applying 1, 3 and 4)
%                  analysis_transform_coding_I.m 
%
%     6- Analysis of transfrom coding results II (loads precomputed results)
%                  analysis_transform_coding_II.m
%
%     7- Analysis of nonlinear ICA results (loads precomputed results)
%                  analysis_nonlinearICA.m
%                   
%  In this script you have a complete example on how to encode and decode
%  a small image. 
%  The analysis of the capabilities of SPCA for component independence
%  require the transform of a lot of data (and hence a lot of time).
%  According to this, only precomputed result will be shown here.
%

%%%%%%%%%%%%%%%%%%%%%%
%%                  %% 
%% TRANSFORM CODING %%
%%                  %%  
%%%%%%%%%%%%%%%%%%%%%%

% (SMALL) IMAGE TO ENCODE: im_o
    load barbara256
    pto=[70 75];
    im_o=Im(pto(1):pto(1)+99,pto(2):pto(2)+99);
    
    im_o = im_o(13:30,18:35); % 18*18 image
    
% INITIALIZATION OF SPCA (BOTH CRITERIA) -it takes about 550 secs/criterion- 

    initialization_4d_texture_data
    
% ENCODING-DECODING USING SPCA infomax (use a separate processor for this)

    % Destination folder
    destination_folder='/media/disk/vista/Papers/aranyas/general_y_analiticas/general/TNN_2012_reproducible/exp_metric_ICA_VQ/image_coding_real/data_coding/'; 
    
    % Transform (it takes about 15 hours! -using a single processor-)
    transform_image_data_criterion_2(im_o,destination_folder); 
    
    % Quantization and Inverse (it takes about 2 hours -using a single processor-)
    rate_factor=0.1222;  % (reduction) factor to be applied to the number of quantization bins given in the initialization
    im2 = quantize_and_invert_image_transf_crit_2(rate_factor,destination_folder,'transformed_4d_image_crit_2',destination_folder);

% ENCODING-DECODING USING SPCA error minimization (use a separate processor for this)

    % Destination folder
    destination_folder='/media/disk/vista/Papers/aranyas/general_y_analiticas/general/TNN_2012_reproducible/exp_metric_ICA_VQ/image_coding_real/data_coding/'; 
    
    % Transform (it takes 20 hours! -using a single processor-)
    transform_image_data_criterion_3(im_o,destination_folder); 
    
    % Quantization and Inverse (it takes about 2 hours -using a single processor-)
    rate_factor=0.1222;  % (reduction) factor to be applied to the number of quantization bins given in the initialization
    im3 = quantize_and_invert_image_transf_crit_3(rate_factor,destination_folder,'transformed_4d_image_crit_3',destination_folder);    
    
% ANALYSIS OF RESULTS (once the encoding-decoding processes are done) 

    analysis_transform_coding_I
    
% PRECOMPUTED CODING RESULTS

    analysis_transform_coding_II

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                     %% 
%% INDEPENDENCE ANALYSIS (PRECOMPUTED) %%
%%                                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    analysis_nonlinearICA
    
    